#!/usr/bin/env node
import minimist from '../node_modules/minimist/index.mjs';
import { cleanUpSinglePath } from './feature/__tests__/cleanForTests.mjs';
import { createPath } from './util/createPath.mjs';
import { defaultRepositoryMapFileConfig } from './feature/config/const.mjs';
import { getTemplateConfig } from './feature/config/defaultTemplateConfig.mjs';
import { bumpVersion } from './feature/prepareTemplate/bumpVersion.mjs';
import { cleanUpTemplate } from './feature/prepareTemplate/cleanUpTemplate.mjs';
import { prepareFileList } from './feature/prepareTemplate/prepareFileList.mjs';
import { scanProjectFolder } from './feature/prepareTemplate/scanProjectFolder.mjs';
import { updateTemplateConfig } from './feature/prepareTemplate/updateTemplateConfig.mjs';
import { createCatalog } from './util/createCatalog.mjs';
import { createFile } from './util/createFile.mjs';
import { debugFunction } from './util/debugFunction.mjs';
import { formatJsonWithPrettier } from './util/formatPrettier.mjs';
import { isFileOrFolderExists } from './util/isFileOrFolderExists.mjs';

const initPrepareTemplate = async (templateConfig) => {
    await cleanUpSinglePath({
        path: createPath([templateConfig.projectCatalog, 'tools']),
        isDebug: templateConfig.isDebug,
    });
    await createFile({
        filePath: createPath([templateConfig.projectCatalog, 'tools', 'test.sh']),
        options: {
            createFolder: true,
        },
        content: 'lorem ipsum dolor sit amet, consectetur adipis',
    });
    if (process.env.STYPE === 'template') {
        await cleanUpSinglePath({
            path: templateConfig.templateCatalogPath,
            isDebug: templateConfig.isDebug,
        });
    }
    if (process.env.STYPE === 'templateRebuild') {
        await createFile({
            filePath: createPath([templateConfig.projectCatalog, 'tools', 'test-new.sh']),
            options: {
                createFolder: true,
            },
            content: 'lorem ipsum dolor sit amet, consectetur adipis new SH',
        });
    }
    return;
};
const init = async (args) => {
    const templateConfig = getTemplateConfig(args);
    const { isDebug, templateCatalogPath, repositoryMapFilePath } = templateConfig;
    debugFunction(isDebug, '=== Start prepare template ===', '[PrepareTemplate]');
    if (process.env.SDEBUG === 'true') {
        await initPrepareTemplate(templateConfig);
    }
    await createCatalog(templateCatalogPath);
    if (!(await isFileOrFolderExists({
        isDebug,
        filePath: repositoryMapFilePath,
    }))) {
        debugFunction(isDebug, { fileExist: repositoryMapFilePath, templateConfig }, '[PrepareTemplate]');
        await createFile({
            filePath: repositoryMapFilePath,
            content: JSON.stringify({
                ...defaultRepositoryMapFileConfig,
                ...templateConfig,
            }),
            isDebug,
            options: {
                overwriteFile: true,
            },
        });
        templateConfig.bumpVersion = false;
    }
    debugFunction(isDebug, templateConfig, '[PrepareTemplate] END init');
    return { templateConfig };
};
const args = minimist(process.argv.slice(2));
let finalConfig = {
    isDebug: false,
};
init(args)
    .then(({ templateConfig }) => {
    finalConfig = templateConfig;
    return bumpVersion(templateConfig);
})
    .then(({ templateConfig }) => {
    finalConfig = templateConfig;
    return cleanUpTemplate(templateConfig);
})
    .then(({ templateConfig }) => {
    finalConfig = templateConfig;
    return scanProjectFolder(templateConfig);
})
    .then(({ templateConfig, templateFileList }) => {
    finalConfig = templateConfig;
    return prepareFileList({ templateConfig, templateFileList });
})
    .then(({ redOnlyFileList, templateConfig, templateFileList, fileList, rootPathFileList }) => {
    finalConfig = templateConfig;
    return updateTemplateConfig({
        templateConfig,
        fileList,
        templateFileList,
        rootPathFileList,
        redOnlyFileList,
    });
})
    .then(async ({ templateConfig }) => {
    await formatJsonWithPrettier(templateConfig.repositoryMapFilePath);
})
    .finally(() => {
    debugFunction(finalConfig?.isDebug, { finalConfig }, '[PrepareTemplate] final config');
    debugFunction(finalConfig?.isDebug, '=== Final prepare template ===', '[PrepareTemplate]');
});

export { init, initPrepareTemplate };
