import { createPath } from './createPath.mjs';
import { readFile } from './readFile.mjs';

/**
 * Checks if a file path matches a gitignore-style pattern
 */
function matchesPattern(filePath, pattern) {
    // Convert gitignore pattern to regex
    const regexPattern = pattern
        .replace(/\./g, '\\.') // Escape dots
        .replace(/\*/g, '.*') // Convert * to .*
        .replace(/\?/g, '.'); // Convert ? to .
    const regex = new RegExp(`^${regexPattern}$`);
    // For extension patterns like "*.json", also check against just the filename
    if (pattern.startsWith('*.')) {
        const fileName = filePath.split('/').pop() || '';
        return regex.test(fileName);
    }
    return regex.test(filePath);
}
/**
 * Reads .sumignore file and returns list of ignored files based on patterns
 * @param params Configuration parameters including project config and file map
 * @returns Array of ignored file paths
 */
async function getIgnoredFilesFromConfig({ config, sumFileMapConfig, }) {
    if (!sumFileMapConfig.sumFileMap) {
        return [];
    }
    const sumIgnoreConfig = await readFile(createPath([config.sumIgnoreFilePath])).then(async (bufferData) => {
        const content = bufferData.toString();
        if (content === '')
            return null;
        return content.split('\n');
    });
    if (!sumIgnoreConfig) {
        return [];
    }
    const ignoredFiles = [];
    for (const ignorePattern of sumIgnoreConfig) {
        const trimmedPattern = ignorePattern.trim();
        if (!trimmedPattern || trimmedPattern.startsWith('#'))
            continue;
        if (trimmedPattern.endsWith('/')) {
            // Handle directory patterns
            const basePattern = trimmedPattern.replace(/\/$/, '');
            Object.keys(sumFileMapConfig.sumFileMap).forEach((filePath) => {
                if (filePath.startsWith(basePattern + '/')) {
                    ignoredFiles.push(filePath);
                }
            });
        }
        else {
            // Handle file patterns (including wildcards)
            Object.keys(sumFileMapConfig.sumFileMap).forEach((filePath) => {
                if (matchesPattern(filePath, trimmedPattern)) {
                    ignoredFiles.push(filePath);
                }
            });
        }
    }
    return [...new Set(ignoredFiles)]; // Remove duplicates
}

export { getIgnoredFilesFromConfig };
